@@ -13,7 +13,10 @@ from neurodiffeq.generators import GeneratorSpherical
 from neurodiffeq.generators import SamplerGenerator
 from neurodiffeq.generators import Generator1D
 from neurodiffeq.generators import Generator2D
+from neurodiffeq.generators import GeneratorND
 from neurodiffeq.function_basis import RealSphericalHarmonics
+from neurodiffeq.neurodiffeq import safe_diff as diff
+from .losses import _losses
 
 
 def _requires_closure(optimizer):
@@ -158,9 +161,11 @@ class BaseSolver(ABC):
         self.optimizer = optimizer if optimizer else Adam(chain.from_iterable(n.parameters() for n in self.nets))
 
         if criterion is None:
-            self.criterion = lambda r: (r ** 2).mean()
+            self.criterion = lambda r, f, x: (r ** 2).mean()
         elif isinstance(criterion, nn.modules.loss._Loss):
-            self.criterion = lambda r: criterion(r, torch.zeros_like(r))
+            self.criterion = lambda r, f, x: criterion(r, torch.zeros_like(r))
+        elif isinstance(criterion, str):
+            self.criterion = _losses[criterion.lower()]
         else:
             self.criterion = criterion
 
@@ -174,7 +179,7 @@ class BaseSolver(ABC):
         # number of batches for training / validation;
         self.n_batches = make_pair_dict(train=n_batches_train, valid=n_batches_valid)
         # current batch of samples, kept for additional_loss term to use
-        self._batch_examples = make_pair_dict()
+        self._batch = make_pair_dict()
         # current network with lowest loss
         self.best_nets = None
         # current lowest loss
@@ -198,6 +203,18 @@ class BaseSolver(ABC):
         """
         return len(self.metrics_history['train_loss'])
 
+    @property
+    def batch(self):
+        return self._batch
+
+    @property
+    def _batch_examples(self):
+        warnings.warn(
+            '`._batch_examples` has been deprecated in favor of `._batch` and will be removed in a future version',
+            FutureWarning,
+        )
+        return self._batch
+
     def compute_func_val(self, net, cond, *coordinates):
         r"""Compute the function value evaluated on the points specified by ``coordinates``.
 
@@ -239,11 +256,11 @@ class BaseSolver(ABC):
         self._update_history(value, metric_type, key='valid')
 
     def _generate_batch(self, key):
-        r"""Generate the next batch, register in self._batch_examples and return the batch.
+        r"""Generate the next batch, register in self._batch and return the batch.
 
         :param key:
             {'train', 'valid'};
-            Dict key in ``self._examples``, ``self._batch_examples``, or ``self._batch_start``
+            Dict key in ``self._examples``, ``self._batch``, or ``self._batch_start``
         :type key: str
         :return: The generated batch of points.
         :type: List[`torch.Tensor`]
@@ -251,15 +268,15 @@ class BaseSolver(ABC):
         # the following side effects are helpful for future extension,
         # especially for additional loss term that depends on the coordinates
         self._phase = key
-        self._batch_examples[key] = [v.reshape(-1, 1) for v in self.generator[key].get_examples()]
-        return self._batch_examples[key]
+        self._batch[key] = [v.reshape(-1, 1) for v in self.generator[key].get_examples()]
+        return self._batch[key]
 
     def _generate_train_batch(self):
-        r"""Generate the next training batch, register in ``self._batch_examples`` and return."""
+        r"""Generate the next training batch, register in ``self._batch`` and return."""
         return self._generate_batch('train')
 
     def _generate_valid_batch(self):
-        r"""Generate the next validation batch, register in ``self._batch_examples`` and return."""
+        r"""Generate the next validation batch, register in ``self._batch`` and return."""
         return self._generate_batch('valid')
 
     def _do_optimizer_step(self, closure=None):
@@ -313,7 +330,14 @@ class BaseSolver(ABC):
                     metric_values[name] += value
                 residuals = self.diff_eqs(*funcs, *batch)
                 residuals = torch.cat(residuals, dim=1)
-                loss = self.criterion(residuals) + self.additional_loss(funcs, key)
+                try:
+                    loss = self.criterion(residuals, funcs, batch) + self.additional_loss(residuals, funcs, batch)
+                except TypeError as e:
+                    warnings.warn(
+                        "You might need to update your code. "
+                        "Since v0.4.0; both `criterion` and `additional_loss` requires three inputs: "
+                        "`residual`, `funcs`, and `coords`. See documentation for more.", FutureWarning)
+                    raise e
 
                 # accumulate gradients before the current graph is collected as garbage
                 if key == 'train':
@@ -390,7 +414,7 @@ class BaseSolver(ABC):
                           "use a MonitorCallback and pass a list of callbacks instead")
             callbacks = [monitor.to_callback()] + list(callbacks)
         if kwargs:
-            raise ValueError(f'Unknown keyword argument(s): {list(kwargs.keys())}')
+            raise ValueError(f'Unknown keyword argument(s): {list(kwargs.keys())}')  # pragma: no cover
 
         for local_epoch in range(max_epochs):
             # stop training if self._stop_training is set to True by a callback
@@ -492,14 +516,20 @@ class BaseSolver(ABC):
         else:
             raise ValueError(f"unrecognized return_type = {return_type}")
 
-    def additional_loss(self, funcs, key):
+    def additional_loss(self, residual, funcs, coords):
         r"""Additional loss terms for training. This method is to be overridden by subclasses.
-        This method can use any of the internal variables: the current batch, the nets, the conditions, etc.
+        This method can use any of the internal variables: self.nets, self.conditions, self.global_epoch, etc.
 
-        :param funcs: Outputs of the networks after parameterization.
-        :type funcs: list[torch.Tensor]
-        :param key: {'train', 'valid'}; Phase of the epoch, used to access the sample batch, etc.
-        :type key: str
+        :param residual: Residual tensor of differential equation. It has shape (N_SAMPLES, N_EQUATIONS)
+        :type residual: torch.Tensor
+        :param funcs:
+            Outputs of the networks after parameterization.
+            There are ``len(nets)`` entries in total. Each entry is a tensor of shape (N_SAMPLES, N_OUTPUT_UNITS).
+        :type funcs: List[torch.Tensor]
+        :param coords:
+            Inputs to the networks; a.k.a. the spatio-temporal coordinates of the system.
+            There are ``N_COORDS`` entries in total. Each entry is a tensor of shape (N_SAMPLES, 1).
+        :type coords: List[torch.Tensor]
         :return: Additional loss. Must be a ``torch.Tensor`` of empty shape (scalar).
         :rtype: torch.Tensor
         """
@@ -555,7 +585,7 @@ class BaseSolution(ABC):
             # Why did we allow `tf` as an option >_<
             # We should phase this out as soon as possible
             if to_numpy == 'tf' or to_numpy == 'torch':
-                to_numpy = True
+                to_numpy = False
             elif to_numpy == 'np':
                 to_numpy = True
             else:
@@ -571,6 +601,44 @@ class BaseSolution(ABC):
         return us if len(self.nets) > 1 else us[0]
 
 
+class GenericSolution(BaseSolution):
+    def _compute_u(self, net, condition, *coords):
+        return condition.enforce(net, *coords)
+
+
+class GenericSolver(BaseSolver):
+    def get_solution(self, copy=True, best=True):
+        r"""Get a (callable) solution object. See this usage example:
+
+        .. code-block:: python3
+
+            solution = solver.get_solution()
+            point_coords = train_generator.get_examples()
+            value_at_points = solution(point_coords)
+
+        :param copy:
+            Whether to make a copy of the networks so that subsequent training doesn't affect the solution;
+            Defaults to True.
+        :type copy: bool
+        :param best:
+            Whether to return the solution with lowest loss instead of the solution after the last epoch.
+            Defaults to True.
+        :type best: bool
+        :return:
+            A solution object which can be called.
+            To evaluate the solution on certain points,
+            you should pass the coordinates vector(s) to the returned solution.
+        :rtype: BaseSolution
+        """
+        nets = self.best_nets if best else self.nets
+        conditions = self.conditions
+        if copy:
+            nets = deepcopy(nets)
+            conditions = deepcopy(conditions)
+
+        return GenericSolution(nets, conditions)
+
+
 class SolverSpherical(BaseSolver):
     r"""A solver class for solving PDEs in spherical coordinates
 
@@ -787,7 +855,10 @@ class SolutionSphericalHarmonics(SolutionSpherical):
             raise ValueError("harmonics_fn should be specified")
 
         if max_degree is not None:
-            warnings.warn("`max_degree` is DEPRECATED; pass `harmonics_fn` instead, which takes precedence")
+            warnings.warn(
+                "`max_degree` is DEPRECATED; pass `harmonics_fn` instead, which takes precedence",
+                FutureWarning,
+            )
             self.harmonics_fn = RealSphericalHarmonics(max_degree=max_degree)
 
         if harmonics_fn is not None:
@@ -879,7 +950,7 @@ class Solver1D(BaseSolver):
     :type shuffle: bool
     """
 
-    def __init__(self, ode_system, conditions, t_min, t_max,
+    def __init__(self, ode_system, conditions, t_min=None, t_max=None,
                  nets=None, train_generator=None, valid_generator=None, analytic_solutions=None, optimizer=None,
                  criterion=None, n_batches_train=1, n_batches_valid=4, metrics=None, n_output_units=1,
                  # deprecated arguments are listed below
@@ -956,6 +1027,198 @@ class Solver1D(BaseSolver):
         return available_variables
 
 
+class BundleSolution1D(BaseSolution):
+    def _compute_u(self, net, condition, *ts):
+        return condition.enforce(net, *ts)
+
+
+class BundleSolver1D(BaseSolver):
+    r"""A solver class for solving ODEs (single-input differential equations)
+    , or a bundle of ODEs for different values of its parameters and/or conditions
+
+    :param ode_system:
+        The ODE system to solve, which maps a torch.Tensor or a tuple of torch.Tensors, to a tuple of ODE residuals,
+        both the input and output must have shape (n_samples, 1).
+    :type ode_system: callable
+    :param conditions:
+        List of conditions for each target function.
+    :type conditions: list[`neurodiffeq.conditions.BaseCondition`]
+    :param t_min:
+        Lower bound of input (start time).
+        Ignored if ``train_generator`` and ``valid_generator`` are both set.
+    :type t_min: float, optional
+    :param t_max:
+        Upper bound of input (start time).
+        Ignored if ``train_generator`` and ``valid_generator`` are both set.
+    :type t_max: float, optional
+    :param theta_min:
+        Lower bound of input (parameters and/or conditions). If conditions are included in the bundle,
+        they should be the first values in the tuple.
+        Defaults to None.
+        Ignored if ``train_generator`` and ``valid_generator`` are both set.
+    :type theta_min: float or tuple, optional
+    :param theta_max:
+        Upper bound of input (parameters and/or conditions). If conditions are included in the bundle,
+        they should be the first values in the tuple.
+        Defaults to None.
+        Ignored if ``train_generator`` and ``valid_generator`` are both set.
+    :type theta_max: float or tuple, optional
+    :param nets:
+        List of neural networks for parameterized solution.
+        If provided, length of ``nets`` must equal that of ``conditions``
+    :type nets: list[torch.nn.Module], optional
+    :param train_generator:
+        Generator for sampling training points,
+        which must provide a ``.get_examples()`` method and a ``.size`` field.
+        ``train_generator`` must be specified if ``t_min`` and ``t_max`` are not set.
+    :type train_generator: `neurodiffeq.generators.BaseGenerator`, optional
+    :param valid_generator:
+        Generator for sampling validation points,
+        which must provide a ``.get_examples()`` method and a ``.size`` field.
+        ``valid_generator`` must be specified if ``t_min`` and ``t_max`` are not set.
+    :type valid_generator: `neurodiffeq.generators.BaseGenerator`, optional
+    :param analytic_solutions:
+        Analytical solutions to be compared with neural net solutions.
+        It maps a torch.Tensor to a tuple of function values.
+        Output shape should match that of ``nets``.
+    :type analytic_solutions: callable, optional
+    :param optimizer:
+        Optimizer to be used for training.
+        Defaults to a ``torch.optim.Adam`` instance that trains on all parameters of ``nets``.
+    :type optimizer: ``torch.nn.optim.Optimizer``, optional
+    :param criterion:
+        Function that maps a ODE residual tensor (of shape (-1, 1)) to a scalar loss.
+    :type criterion: callable, optional
+    :param n_batches_train:
+        Number of batches to train in every epoch, where batch-size equals ``train_generator.size``.
+        Defaults to 1.
+    :type n_batches_train: int, optional
+    :param n_batches_valid:
+        Number of batches to validate in every epoch, where batch-size equals ``valid_generator.size``.
+        Defaults to 4.
+    :type n_batches_valid: int, optional
+    :param metrics:
+        Additional metrics to be logged (besides loss). ``metrics`` should be a dict where
+
+        - Keys are metric names (e.g. 'analytic_mse');
+        - Values are functions (callables) that computes the metric value.
+          These functions must accept the same input as the differential equation ``ode_system``.
+
+    :type metrics: dict[str, callable], optional
+    :param n_output_units:
+        Number of output units for each neural network.
+        Ignored if ``nets`` is specified.
+        Defaults to 1.
+    :type n_output_units: int, optional
+    :param batch_size:
+        **[DEPRECATED and IGNORED]**
+        Each batch will use all samples generated.
+        Please specify ``n_batches_train`` and ``n_batches_valid`` instead.
+    :type batch_size: int
+    :param shuffle:
+        **[DEPRECATED and IGNORED]**
+        Shuffling should be performed by generators.
+    :type shuffle: bool
+    """
+
+    def __init__(self, ode_system, conditions, t_min, t_max,
+                 theta_min=None, theta_max=None,
+                 nets=None, train_generator=None, valid_generator=None, analytic_solutions=None, optimizer=None,
+                 criterion=None, n_batches_train=1, n_batches_valid=4, metrics=None, n_output_units=1,
+                 # deprecated arguments are listed below
+                 batch_size=None, shuffle=None):
+
+        if train_generator is None or valid_generator is None:
+            if t_min is None or t_max is None:
+                raise ValueError(f"Either generator is not provided, t_min and t_max should be both provided: \n"
+                                 f"got t_min={t_min}, t_max={t_max}, "
+                                 f"train_generator={train_generator}, valid_generator={valid_generator}")
+
+        if isinstance(theta_min, float) or isinstance(theta_min, int):
+            r_min = (t_min,) + (theta_min,)
+
+        if isinstance(theta_max, float) or isinstance(theta_max, int):
+            r_max = (t_max,) + (theta_max,)
+
+        if theta_min is None and theta_max is None:
+            r_min = (t_min,)
+            r_max = (t_max,)
+
+        if isinstance(theta_min, tuple) and isinstance(theta_max, tuple):
+            r_min = (t_min,) + theta_min
+            r_max = (t_max,) + theta_max
+
+        n_input_units = len(r_min)
+
+        methods = ['equally-spaced' for i in range(n_input_units)]
+
+        grid = tuple(32 for j in range(n_input_units))
+
+        if train_generator is None:
+            train_generator = GeneratorND(grid, r_min=r_min, r_max=r_max, methods=methods, noisy=True)
+        if valid_generator is None:
+            valid_generator = GeneratorND(grid, r_min=r_min, r_max=r_max, methods=methods, noisy=False)
+
+        self.r_min, self.r_max = r_min, r_max
+
+        super(BundleSolver1D, self).__init__(
+            diff_eqs=ode_system,
+            conditions=conditions,
+            nets=nets,
+            train_generator=train_generator,
+            valid_generator=valid_generator,
+            analytic_solutions=analytic_solutions,
+            optimizer=optimizer,
+            criterion=criterion,
+            n_batches_train=n_batches_train,
+            n_batches_valid=n_batches_valid,
+            metrics=metrics,
+            n_input_units=n_input_units,
+            n_output_units=n_output_units,
+            shuffle=shuffle,
+            batch_size=batch_size,
+        )
+
+    def get_solution(self, copy=True, best=True):
+        r"""Get a (callable) solution object. See this usage example:
+
+        .. code-block:: python3
+
+            solution = solver.get_solution()
+            point_coords = train_generator.get_examples()
+            value_at_points = solution(point_coords)
+
+        :param copy:
+            Whether to make a copy of the networks so that subsequent training doesn't affect the solution;
+            Defaults to True.
+        :type copy: bool
+        :param best:
+            Whether to return the solution with lowest loss instead of the solution after the last epoch.
+            Defaults to True.
+        :type best: bool
+        :return:
+            A solution object which can be called.
+            To evaluate the solution on certain points,
+            you should pass the coordinates vector(s) to the returned solution.
+        :rtype: BaseSolution
+        """
+        nets = self.best_nets if best else self.nets
+        conditions = self.conditions
+        if copy:
+            nets = deepcopy(nets)
+            conditions = deepcopy(conditions)
+
+        return BundleSolution1D(nets, conditions)
+
+    def _get_internal_variables(self):
+        available_variables = super(BundleSolver1D, self)._get_internal_variables()
+        available_variables.update({
+            'r_min': self.r_min,
+            'r_max': self.r_max,
+        })
+        return available_variables
+
+
 class Solution2D(BaseSolution):
     def _compute_u(self, net, condition, xs, ys):
         return condition.enforce(net, xs, ys)
@@ -1042,7 +1305,7 @@ class Solver2D(BaseSolver):
     :type shuffle: bool
     """
 
-    def __init__(self, pde_system, conditions, xy_min, xy_max,
+    def __init__(self, pde_system, conditions, xy_min=None, xy_max=None,
                  nets=None, train_generator=None, valid_generator=None, analytic_solutions=None, optimizer=None,
                  criterion=None, n_batches_train=1, n_batches_valid=4, metrics=None, n_output_units=1,
                  # deprecated arguments are listed below
